@@ -28,8 +28,8 @@
 //! ```
 //!
 //! [`Keys`]: Key
+
 mod key;
-use alloc::boxed::Box;
 
 use alloc::vec::Vec;
 use core::ops::Deref;
@@ -37,6 +37,7 @@ use core::ops::Deref;
 pub use key::Key;
 
 use crate::parser::ParserError;
+use crate::parser::SubtagIterator;
 
 /// A list of [`Private Use Extensions`] as defined in [`Unicode Locale
 /// Identifier`] specification.
@@ -61,7 +62,7 @@ use crate::parser::ParserError;
 /// [`Private Use Extensions`]: https://unicode.org/reports/tr35/#pu_extensions
 /// [`Unicode Locale Identifier`]: https://unicode.org/reports/tr35/#Unicode_locale_identifier
 #[derive(Clone, PartialEq, Eq, Debug, Default, Hash, PartialOrd, Ord)]
-pub struct Private(Option<Box<[Key]>>);
+pub struct Private(Vec<Key>);
 
 impl Private {
     /// Returns a new empty list of private-use extensions. Same as [`default()`](Default::default()), but is `const`.
@@ -75,7 +76,7 @@ impl Private {
     /// ```
     #[inline]
     pub const fn new() -> Self {
-        Self(None)
+        Self(Vec::new())
     }
 
     /// A constructor which takes a pre-sorted list of [`Key`].
@@ -94,11 +95,7 @@ impl Private {
     /// assert_eq!(&private.to_string(), "-x-foo-bar");
     /// ```
     pub fn from_vec_unchecked(input: Vec<Key>) -> Self {
-        if input.is_empty() {
-            Self(None)
-        } else {
-            Self(Some(input.into_boxed_slice()))
-        }
+        Self(input)
     }
 
     /// Empties the [`Private`] list.
@@ -121,16 +118,25 @@ impl Private {
     /// assert_eq!(&private.to_string(), "");
     /// ```
     pub fn clear(&mut self) {
-        self.0 = None;
+        self.0.clear();
     }
 
-    pub(crate) fn try_from_iter<'a>(
-        iter: &mut impl Iterator<Item = &'a [u8]>,
-    ) -> Result<Self, ParserError> {
+    pub(crate) fn try_from_iter(iter: &mut SubtagIterator) -> Result<Self, ParserError> {
         let keys = iter.map(Key::from_bytes).collect::<Result<Vec<_>, _>>()?;
 
         Ok(Self::from_vec_unchecked(keys))
     }
+
+    pub(crate) fn for_each_subtag_str<E, F>(&self, f: &mut F) -> Result<(), E>
+    where
+        F: FnMut(&str) -> Result<(), E>,
+    {
+        if self.is_empty() {
+            return Ok(());
+        }
+        f("x")?;
+        self.deref().iter().map(|t| t.as_str()).try_for_each(f)
+    }
 }
 
 impl core::fmt::Display for Private {
@@ -168,10 +174,6 @@ impl Deref for Private {
     type Target = [Key];
 
     fn deref(&self) -> &Self::Target {
-        if let Some(ref data) = self.0 {
-            data
-        } else {
-            &[]
-        }
+        self.0.deref()
     }
 }
