@@ -2,11 +2,9 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
-use alloc::boxed::Box;
-
-use alloc::vec::Vec;
 use core::borrow::Borrow;
-use core::ops::Deref;
+use core::iter::FromIterator;
+use litemap::LiteMap;
 
 use super::Key;
 use super::Value;
@@ -32,12 +30,12 @@ use super::Value;
 ///     .expect("Failed to parse a Key.");
 /// let value: Value = "hybrid".parse()
 ///     .expect("Failed to parse a Value.");
-/// let fields = Fields::from_vec_unchecked(vec![(key, value)]);
+/// let fields: Fields = vec![(key, value)].into_iter().collect();
 ///
 /// assert_eq!(&fields.to_string(), "h0-hybrid");
 /// ```
 #[derive(Clone, PartialEq, Eq, Debug, Default, Hash, PartialOrd, Ord)]
-pub struct Fields(Option<Box<[(Key, Value)]>>);
+pub struct Fields(LiteMap<Key, Value>);
 
 impl Fields {
     /// Returns a new empty list of key-value pairs. Same as [`default()`](Default::default()), but is `const`.
@@ -51,31 +49,25 @@ impl Fields {
     /// ```
     #[inline]
     pub const fn new() -> Self {
-        Self(None)
+        Self(LiteMap::new())
     }
 
-    /// A constructor which takes a pre-sorted list of `(`[`Key`]`, `[`Value`]`)` tuples.
-    ///
+    /// Returns `true` if there are no fields.
     ///
     /// # Examples
     ///
     /// ```
-    /// use icu::locid::extensions::transform::{Fields, Key, Value};
+    /// use icu::locid::Locale;
+    /// use icu::locid::extensions::transform::Fields;
     ///
-    /// let key: Key = "h0".parse()
-    ///     .expect("Failed to parse a Key.");
-    /// let value: Value = "hybrid".parse()
-    ///     .expect("Failed to parse a Value.");
-    /// let fields = Fields::from_vec_unchecked(vec![(key, value)]);
+    /// let loc1 = Locale::from_bytes(b"und-t-h0-hybrid").unwrap();
+    /// let loc2 = Locale::from_bytes(b"und-u-ca-buddhist").unwrap();
     ///
-    /// assert_eq!(&fields.to_string(), "h0-hybrid");
+    /// assert!(!loc1.extensions.transform.fields.is_empty());
+    /// assert!(loc2.extensions.transform.fields.is_empty());
     /// ```
-    pub fn from_vec_unchecked(input: Vec<(Key, Value)>) -> Self {
-        if input.is_empty() {
-            Self(None)
-        } else {
-            Self(Some(input.into_boxed_slice()))
-        }
+    pub fn is_empty(&self) -> bool {
+        self.0.is_empty()
     }
 
     /// Empties the [`Fields`] list.
@@ -89,7 +81,7 @@ impl Fields {
     ///     .expect("Failed to parse a Key.");
     /// let value: Value = "hybrid".parse()
     ///     .expect("Failed to parse a Value.");
-    /// let mut fields = Fields::from_vec_unchecked(vec![(key, value)]);
+    /// let mut fields: Fields = vec![(key, value)].into_iter().collect();
     ///
     /// assert_eq!(&fields.to_string(), "h0-hybrid");
     ///
@@ -98,7 +90,7 @@ impl Fields {
     /// assert_eq!(&fields.to_string(), "");
     /// ```
     pub fn clear(&mut self) {
-        self.0 = None;
+        self.0.clear();
     }
 
     /// Returns `true` if the list contains a [`Value`] for the specified [`Key`].
@@ -113,18 +105,18 @@ impl Fields {
     ///     .expect("Failed to parse a Key.");
     /// let value: Value = "hybrid".parse()
     ///     .expect("Failed to parse a Value.");
-    /// let mut fields = Fields::from_vec_unchecked(vec![(key, value)]);
+    /// let mut fields: Fields = vec![(key, value)].into_iter().collect();
     ///
     /// let key: Key = "h0".parse()
     ///     .expect("Failed to parse a Key.");
     /// assert!(&fields.contains_key(&key));
     /// ```
-    pub fn contains_key<Q>(&self, key: Q) -> bool
+    pub fn contains_key<Q>(&self, key: &Q) -> bool
     where
-        Q: Borrow<Key>,
+        Key: Borrow<Q>,
+        Q: Ord,
     {
-        self.binary_search_by_key(key.borrow(), |(key, _)| *key)
-            .is_ok()
+        self.0.contains_key(key)
     }
 
     /// Returns a reference to the [`Value`] corresponding to the [`Key`].
@@ -139,7 +131,7 @@ impl Fields {
     ///     .expect("Failed to parse a Key.");
     /// let value: Value = "hybrid".parse()
     ///     .expect("Failed to parse a Value.");
-    /// let mut fields = Fields::from_vec_unchecked(vec![(key, value)]);
+    /// let mut fields: Fields = vec![(key, value)].into_iter().collect();
     ///
     /// let key: Key = "h0".parse()
     ///     .expect("Failed to parse a Key.");
@@ -148,28 +140,95 @@ impl Fields {
     ///     Some("hybrid".to_string())
     /// );
     /// ```
-    pub fn get<Q>(&self, key: Q) -> Option<&Value>
+    pub fn get<Q>(&self, key: &Q) -> Option<&Value>
     where
-        Q: Borrow<Key>,
+        Key: Borrow<Q>,
+        Q: Ord,
     {
-        if let Ok(idx) = self.binary_search_by_key(key.borrow(), |(key, _)| *key) {
-            self.deref().get(idx).map(|(_, v)| v)
-        } else {
-            None
+        self.0.get(key)
+    }
+
+    /// Sets the specified keyword, returning the old value if it already existed.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use std::str::FromStr;
+    /// use std::string::ToString;
+    /// use icu::locid::transform_ext_key;
+    /// use icu::locid::Locale;
+    /// use icu::locid::extensions::transform::Key;
+    /// use icu::locid::extensions::transform::Value;
+    ///
+    /// const D0_KEY: Key = transform_ext_key!("d0");
+    /// let lower = Value::from_str("lower").expect("valid extension subtag");
+    /// let casefold = Value::from_str("casefold").expect("valid extension subtag");
+    ///
+    /// let mut loc: Locale = "en-t-hi-d0-casefold".parse()
+    ///     .expect("valid BCP-47 identifier");
+    /// let old_value = loc.extensions.transform.fields.set(
+    ///     D0_KEY,
+    ///     lower
+    /// );
+    ///
+    /// assert_eq!(old_value, Some(casefold));
+    /// assert_eq!(loc, "en-t-hi-d0-lower");
+    /// ```
+    pub fn set(&mut self, key: Key, value: Value) -> Option<Value> {
+        self.0.insert(key, value)
+    }
+
+    /// Retains a subset of fields as specified by the predicate function.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use icu::locid::Locale;
+    /// use std::str::FromStr;
+    ///
+    /// let mut loc: Locale = "und-t-h0-hybrid-d0-hex-m0-xml".parse().unwrap();
+    ///
+    /// loc.extensions.transform.fields.retain_by_key(|k| k == "h0");
+    /// assert_eq!(loc, "und-t-h0-hybrid");
+    ///
+    /// loc.extensions.transform.fields.retain_by_key(|k| k == "d0");
+    /// assert_eq!(loc, "und");
+    /// ```
+    pub fn retain_by_key<F>(&mut self, mut predicate: F)
+    where
+        F: FnMut(&Key) -> bool,
+    {
+        self.0.retain(|k, _| predicate(k))
+    }
+
+    pub(crate) fn for_each_subtag_str<E, F>(&self, f: &mut F) -> Result<(), E>
+    where
+        F: FnMut(&str) -> Result<(), E>,
+    {
+        for (k, v) in self.0.iter() {
+            f(k.as_str())?;
+            v.for_each_subtag_str(f)?;
         }
+        Ok(())
+    }
+
+    /// This needs to be its own method to help with type inference in helpers.rs
+    #[cfg(test)]
+    pub(crate) fn from_tuple_vec(v: Vec<(Key, Value)>) -> Self {
+        v.into_iter().collect()
+    }
+}
+
+impl From<LiteMap<Key, Value>> for Fields {
+    fn from(map: LiteMap<Key, Value>) -> Self {
+        Self(map)
+    }
+}
+
+impl FromIterator<(Key, Value)> for Fields {
+    fn from_iter<I: IntoIterator<Item = (Key, Value)>>(iter: I) -> Self {
+        LiteMap::from_iter(iter).into()
     }
 }
 
 impl_writeable_for_key_value!(Fields, "h0", "hybrid", "m0", "m0-true");
-
-impl Deref for Fields {
-    type Target = [(Key, Value)];
-
-    fn deref(&self) -> &Self::Target {
-        if let Some(ref data) = self.0 {
-            data
-        } else {
-            &[]
-        }
-    }
-}
