@@ -27,8 +27,8 @@
 //!     .expect("Parsing value failed.");
 //!
 //! assert_eq!(loc.extensions.transform.lang, Some(lang));
-//! assert!(loc.extensions.transform.fields.contains_key(key));
-//! assert_eq!(loc.extensions.transform.fields.get(key), Some(&value));
+//! assert!(loc.extensions.transform.fields.contains_key(&key));
+//! assert_eq!(loc.extensions.transform.fields.get(&key), Some(&value));
 //!
 //! assert_eq!(&loc.extensions.transform.to_string(), "-t-es-AR-h0-hybrid");
 //! ```
@@ -40,12 +40,12 @@ pub use fields::Fields;
 pub use key::Key;
 pub use value::Value;
 
+use crate::parser::SubtagIterator;
 use crate::parser::{parse_language_identifier_from_iter, ParserError, ParserMode};
 use crate::subtags::Language;
 use crate::LanguageIdentifier;
 use alloc::vec;
-
-use core::iter::Peekable;
+use litemap::LiteMap;
 
 /// A list of [`Unicode BCP47 T Extensions`] as defined in [`Unicode Locale
 /// Identifier`] specification.
@@ -79,6 +79,7 @@ use core::iter::Peekable;
 /// [`Unicode Locale Identifier`]: https://unicode.org/reports/tr35/#Unicode_locale_identifier
 #[derive(Clone, PartialEq, Eq, Debug, Default, Hash, PartialOrd, Ord)]
 #[allow(missing_docs)] // TODO(#1028) - Add missing docs.
+#[allow(clippy::exhaustive_structs)] // spec-backed stable datastructure
 pub struct Transform {
     pub lang: Option<LanguageIdentifier>,
     pub fields: Fields,
@@ -118,11 +119,25 @@ impl Transform {
         self.lang.is_none() && self.fields.is_empty()
     }
 
-    pub(crate) fn try_from_iter<'a>(
-        iter: &mut Peekable<impl Iterator<Item = &'a [u8]>>,
-    ) -> Result<Self, ParserError> {
+    /// Clears the transform extension, effectively removing it from the locale.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use icu::locid::Locale;
+    ///
+    /// let mut loc: Locale = "en-US-t-es-AR".parse().unwrap();
+    /// loc.extensions.transform.clear();
+    /// assert_eq!(loc, "en-US");
+    /// ```
+    pub fn clear(&mut self) {
+        self.lang = None;
+        self.fields.clear();
+    }
+
+    pub(crate) fn try_from_iter(iter: &mut SubtagIterator) -> Result<Self, ParserError> {
         let mut tlang = None;
-        let mut tfields = vec![];
+        let mut tfields = LiteMap::new();
 
         if let Some(subtag) = iter.peek() {
             if Language::from_bytes(subtag).is_ok() {
@@ -144,10 +159,10 @@ impl Transform {
                     if current_tvalue.is_empty() {
                         return Err(ParserError::InvalidExtension);
                     }
-                    tfields.push((
+                    tfields.try_insert(
                         tkey,
                         Value::from_vec_unchecked(current_tvalue.drain(..).flatten().collect()),
-                    ));
+                    );
                     current_tkey = None;
                     continue;
                 }
@@ -164,19 +179,31 @@ impl Transform {
             if current_tvalue.is_empty() {
                 return Err(ParserError::InvalidExtension);
             }
-            tfields.push((
+            tfields.try_insert(
                 tkey,
                 Value::from_vec_unchecked(current_tvalue.into_iter().flatten().collect()),
-            ));
+            );
         }
 
-        tfields.sort_by_key(|f| f.0);
-
         Ok(Self {
             lang: tlang,
-            fields: Fields::from_vec_unchecked(tfields),
+            fields: tfields.into(),
         })
     }
+
+    pub(crate) fn for_each_subtag_str<E, F>(&self, f: &mut F) -> Result<(), E>
+    where
+        F: FnMut(&str) -> Result<(), E>,
+    {
+        if self.is_empty() {
+            return Ok(());
+        }
+        f("t")?;
+        if let Some(lang) = &self.lang {
+            lang.for_each_subtag_str(f)?;
+        }
+        self.fields.for_each_subtag_str(f)
+    }
 }
 
 impl core::fmt::Display for Transform {
