@@ -3,19 +3,18 @@
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
 use crate::parser::{get_subtag_iterator, ParserError};
-use alloc::boxed::Box;
 use alloc::vec;
 use alloc::vec::Vec;
 use core::ops::RangeInclusive;
 use core::str::FromStr;
-use tinystr::TinyStr8;
+use tinystr::TinyAsciiStr;
 
 #[derive(Debug, PartialEq, Eq, Clone, Hash, PartialOrd, Ord)]
 #[allow(missing_docs)] // TODO(#1028) - Add missing docs.
-pub struct Value(Box<[TinyStr8]>);
+pub struct Value(Vec<TinyAsciiStr<{ *TYPE_LENGTH.end() }>>);
 
 const TYPE_LENGTH: RangeInclusive<usize> = 3..=8;
-const TRUE_TVALUE: TinyStr8 = tinystr::tinystr8!("true");
+const TRUE_TVALUE: TinyAsciiStr<8> = tinystr::tinystr!(8, "true");
 
 /// A value used in a list of [`Fields`](super::Fields).
 ///
@@ -61,7 +60,8 @@ impl Value {
                 return Err(ParserError::InvalidExtension);
             }
             has_value = true;
-            let val = TinyStr8::from_bytes(subtag).map_err(|_| ParserError::InvalidExtension)?;
+            let val =
+                TinyAsciiStr::from_bytes(subtag).map_err(|_| ParserError::InvalidExtension)?;
             if val != TRUE_TVALUE {
                 v.push(val);
             }
@@ -70,19 +70,21 @@ impl Value {
         if !has_value {
             return Err(ParserError::InvalidExtension);
         }
-        Ok(Self(v.into_boxed_slice()))
+        Ok(Self(v))
     }
 
-    pub(crate) fn from_vec_unchecked(input: Vec<TinyStr8>) -> Self {
-        Self(input.into_boxed_slice())
+    pub(crate) fn from_vec_unchecked(input: Vec<TinyAsciiStr<{ *TYPE_LENGTH.end() }>>) -> Self {
+        Self(input)
     }
 
     pub(crate) fn is_type_subtag(t: &[u8]) -> bool {
         TYPE_LENGTH.contains(&t.len()) && !t.iter().any(|c: &u8| !c.is_ascii_alphanumeric())
     }
 
-    pub(crate) fn parse_subtag(t: &[u8]) -> Result<Option<TinyStr8>, ParserError> {
-        let s = TinyStr8::from_bytes(t).map_err(|_| ParserError::InvalidSubtag)?;
+    pub(crate) fn parse_subtag(
+        t: &[u8],
+    ) -> Result<Option<TinyAsciiStr<{ *TYPE_LENGTH.end() }>>, ParserError> {
+        let s = TinyAsciiStr::from_bytes(t).map_err(|_| ParserError::InvalidSubtag)?;
         if !TYPE_LENGTH.contains(&t.len()) || !s.is_ascii_alphanumeric() {
             return Err(ParserError::InvalidExtension);
         }
@@ -95,6 +97,18 @@ impl Value {
             Ok(Some(s))
         }
     }
+
+    pub(crate) fn for_each_subtag_str<E, F>(&self, f: &mut F) -> Result<(), E>
+    where
+        F: FnMut(&str) -> Result<(), E>,
+    {
+        if self.0.is_empty() {
+            f("true")?;
+        } else {
+            self.0.iter().map(TinyAsciiStr::as_str).try_for_each(f)?;
+        }
+        Ok(())
+    }
 }
 
 impl FromStr for Value {
