@@ -5,7 +5,7 @@
 use core::str::FromStr;
 
 use crate::parser::errors::ParserError;
-use tinystr::TinyStr4;
+use tinystr::TinyAsciiStr;
 
 /// A key used in a list of [`Keywords`](super::Keywords).
 ///
@@ -24,7 +24,7 @@ use tinystr::TinyStr4;
 /// assert_eq!(key1, "ca");
 /// ```
 #[derive(Debug, PartialEq, Eq, Clone, Hash, PartialOrd, Ord, Copy)]
-pub struct Key(TinyStr4);
+pub struct Key(TinyAsciiStr<KEY_LENGTH>);
 
 const KEY_LENGTH: usize = 2;
 
@@ -42,7 +42,8 @@ impl Key {
     ///
     /// assert_eq!(key, "ca");
     /// ```
-    pub fn from_bytes(key: &[u8]) -> Result<Self, ParserError> {
+    pub const fn from_bytes(key: &[u8]) -> Result<Self, ParserError> {
+        #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
         if key.len() != KEY_LENGTH
             || !key[0].is_ascii_alphanumeric()
             || !key[1].is_ascii_alphabetic()
@@ -50,24 +51,27 @@ impl Key {
             return Err(ParserError::InvalidExtension);
         }
 
-        let key = TinyStr4::from_bytes(key).map_err(|_| ParserError::InvalidSubtag)?;
+        let key = match TinyAsciiStr::from_bytes(key) {
+            Ok(k) => k,
+            Err(_) => return Err(ParserError::InvalidSubtag),
+        };
         Ok(Self(key.to_ascii_lowercase()))
     }
 
-    /// A constructor which takes a TinyStr4 and produces a [`Key`]
+    /// A constructor which takes a TinyAsciiStr and produces a [`Key`]
     /// without doing any checking.
     ///
     /// # Examples
     ///
     /// ```
     /// use icu::locid::extensions::unicode::Key;
-    /// use tinystr::tinystr4;
+    /// use tinystr::tinystr;
     ///
-    /// let key = Key::from_tinystr4_unchecked(tinystr4!("ca"));
+    /// let key = Key::from_tinystr_unchecked(tinystr!(2, "ca"));
     ///
     /// assert_eq!(key, "ca");
     /// ```
-    pub fn from_tinystr4_unchecked(key: TinyStr4) -> Self {
+    pub fn from_tinystr_unchecked(key: TinyAsciiStr<KEY_LENGTH>) -> Self {
         Self(key)
     }
 
@@ -101,6 +105,12 @@ impl FromStr for Key {
 
 impl_writeable_for_single_subtag!(Key, "ca");
 
+impl PartialEq<str> for Key {
+    fn eq(&self, other: &str) -> bool {
+        self.as_str() == other
+    }
+}
+
 impl PartialEq<&str> for Key {
     fn eq(&self, other: &&str) -> bool {
         self.as_str() == *other
