@@ -43,8 +43,8 @@ pub use keywords::Keywords;
 pub use value::Value;
 
 use crate::parser::ParserError;
-
-use core::iter::Peekable;
+use crate::parser::SubtagIterator;
+use litemap::LiteMap;
 
 /// Unicode Extensions provide information about user preferences in a given locale.
 ///
@@ -70,11 +70,12 @@ use core::iter::Peekable;
 ///
 /// let key: Key = "ca".parse().expect("Parsing key failed.");
 /// let value: Value = "buddhist".parse().expect("Parsing value failed.");
-/// assert_eq!(loc.extensions.unicode.keywords.get(key),
+/// assert_eq!(loc.extensions.unicode.keywords.get(&key),
 ///            Some(&value));
 /// ```
 #[derive(Clone, PartialEq, Eq, Debug, Default, Hash, PartialOrd, Ord)]
 #[allow(missing_docs)] // TODO(#1028) - Add missing docs.
+#[allow(clippy::exhaustive_structs)] // spec-backed stable datastructure
 pub struct Unicode {
     pub keywords: Keywords,
     pub attributes: Attributes,
@@ -114,11 +115,27 @@ impl Unicode {
         self.keywords.is_empty() && self.attributes.is_empty()
     }
 
-    pub(crate) fn try_from_iter<'a>(
-        iter: &mut Peekable<impl Iterator<Item = &'a [u8]>>,
-    ) -> Result<Self, ParserError> {
+    /// Clears all Unicode extension keywords and attributes, effectively removing
+    /// the Unicode extension.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use std::str::FromStr;
+    /// use icu::locid::Locale;
+    ///
+    /// let mut loc: Locale = "und-t-mul-u-hello-ca-buddhist-hc-h12".parse().unwrap();
+    /// loc.extensions.unicode.clear();
+    /// assert_eq!(loc, "und-t-mul");
+    /// ```
+    pub fn clear(&mut self) {
+        self.keywords.clear();
+        self.attributes.clear();
+    }
+
+    pub(crate) fn try_from_iter(iter: &mut SubtagIterator) -> Result<Self, ParserError> {
         let mut attributes = vec![];
-        let mut keywords = vec![];
+        let mut keywords = LiteMap::new();
 
         let mut current_keyword = None;
         let mut current_type = vec![];
@@ -138,9 +155,7 @@ impl Unicode {
             let slen = subtag.len();
             if slen == 2 {
                 if let Some(kw) = current_keyword.take() {
-                    if let Err(idx) = keywords.binary_search_by_key(&kw, |(k, _)| *k) {
-                        keywords.insert(idx, (kw, Value::from_vec_unchecked(current_type)));
-                    }
+                    keywords.try_insert(kw, Value::from_vec_unchecked(current_type));
                     current_type = vec![];
                 }
                 current_keyword = Some(Key::from_bytes(subtag)?);
@@ -157,9 +172,7 @@ impl Unicode {
         }
 
         if let Some(kw) = current_keyword.take() {
-            if let Err(idx) = keywords.binary_search_by_key(&kw, |(k, _)| *k) {
-                keywords.insert(idx, (kw, Value::from_vec_unchecked(current_type)));
-            }
+            keywords.try_insert(kw, Value::from_vec_unchecked(current_type));
         }
 
         // Ensure we've defined at least one attribute or keyword
@@ -168,10 +181,23 @@ impl Unicode {
         }
 
         Ok(Self {
-            keywords: Keywords::from_vec_unchecked(keywords),
+            keywords: keywords.into(),
             attributes: Attributes::from_vec_unchecked(attributes),
         })
     }
+
+    pub(crate) fn for_each_subtag_str<E, F>(&self, f: &mut F) -> Result<(), E>
+    where
+        F: FnMut(&str) -> Result<(), E>,
+    {
+        if self.is_empty() {
+            return Ok(());
+        }
+        f("u")?;
+        self.attributes.for_each_subtag_str(f)?;
+        self.keywords.for_each_subtag_str(f)?;
+        Ok(())
+    }
 }
 
 impl core::fmt::Display for Unicode {
