@@ -2,13 +2,12 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
+use crate::helpers::ShortVec;
 use crate::parser::{get_subtag_iterator, ParserError};
-use alloc::boxed::Box;
-use alloc::vec;
 use alloc::vec::Vec;
 use core::ops::RangeInclusive;
 use core::str::FromStr;
-use tinystr::TinyStr8;
+use tinystr::TinyAsciiStr;
 
 /// A value used in a list of [`Keywords`](super::Keywords).
 ///
@@ -27,15 +26,20 @@ use tinystr::TinyStr8;
 ///     .expect("Failed to parse a Value.");
 /// let value2: Value = "islamic-civil".parse()
 ///     .expect("Failed to parse a Value.");
+/// let value3: Value = "true".parse()
+///     .expect("Failed to parse a Value.");
 ///
 /// assert_eq!(&value1.to_string(), "gregory");
 /// assert_eq!(&value2.to_string(), "islamic-civil");
+///
+/// // The value "true" is special-cased to an empty value
+/// assert_eq!(&value3.to_string(), "");
 /// ```
 #[derive(Debug, PartialEq, Eq, Clone, Hash, PartialOrd, Ord)]
-pub struct Value(Box<[TinyStr8]>);
+pub struct Value(ShortVec<TinyAsciiStr<{ *VALUE_LENGTH.end() }>>);
 
 const VALUE_LENGTH: RangeInclusive<usize> = 3..=8;
-const TRUE_VALUE: TinyStr8 = tinystr::tinystr8!("true");
+const TRUE_VALUE: TinyAsciiStr<8> = tinystr::tinystr!(8, "true");
 
 impl Value {
     /// A constructor which takes a utf8 slice, parses it and
@@ -52,33 +56,72 @@ impl Value {
     /// assert_eq!(&value.to_string(), "buddhist");
     /// ```
     pub fn from_bytes(input: &[u8]) -> Result<Self, ParserError> {
-        let mut v = vec![];
+        let mut v = ShortVec::new();
 
         if !input.is_empty() {
             for subtag in get_subtag_iterator(input) {
-                if !Self::is_type_subtag(subtag) {
-                    return Err(ParserError::InvalidExtension);
-                }
-                let val =
-                    TinyStr8::from_bytes(subtag).map_err(|_| ParserError::InvalidExtension)?;
-                if val != TRUE_VALUE {
+                let val = Self::subtag_from_bytes(subtag)?;
+                if let Some(val) = val {
                     v.push(val);
                 }
             }
         }
-        Ok(Self(v.into_boxed_slice()))
+        Ok(Self(v))
     }
 
-    pub(crate) fn from_vec_unchecked(input: Vec<TinyStr8>) -> Self {
-        Self(input.into_boxed_slice())
+    /// Const constructor for when the value contains only a single subtag.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use icu::locid::extensions::unicode::Value;
+    ///
+    /// Value::try_from_single_subtag(b"buddhist").expect("valid subtag");
+    /// Value::try_from_single_subtag(b"#####").expect_err("invalid subtag");
+    /// Value::try_from_single_subtag(b"foo-bar").expect_err("not a single subtag");
+    /// ```
+    pub const fn try_from_single_subtag(subtag: &[u8]) -> Result<Self, ParserError> {
+        match Self::subtag_from_bytes(subtag) {
+            Err(_) => Err(ParserError::InvalidExtension),
+            Ok(option) => Ok(Self::from_tinystr(option)),
+        }
     }
 
-    pub(crate) fn is_type_subtag(t: &[u8]) -> bool {
-        VALUE_LENGTH.contains(&t.len()) && !t.iter().any(|c: &u8| !c.is_ascii_alphanumeric())
+    #[doc(hidden)]
+    pub fn as_tinystr_slice(&self) -> &[TinyAsciiStr<8>] {
+        self.0.as_slice()
     }
 
-    pub(crate) fn parse_subtag(t: &[u8]) -> Result<Option<TinyStr8>, ParserError> {
-        let s = TinyStr8::from_bytes(t).map_err(|_| ParserError::InvalidSubtag)?;
+    #[doc(hidden)]
+    pub const fn from_tinystr(subtag: Option<TinyAsciiStr<8>>) -> Self {
+        match subtag {
+            None => Self(ShortVec::new()),
+            Some(val) => {
+                debug_assert!(val.is_ascii_alphanumeric());
+                debug_assert!(!matches!(val, TRUE_VALUE));
+                Self(ShortVec::new_single(val))
+            }
+        }
+    }
+
+    pub(crate) fn from_vec_unchecked(input: Vec<TinyAsciiStr<8>>) -> Self {
+        Self(input.into())
+    }
+
+    #[doc(hidden)]
+    pub const fn subtag_from_bytes(bytes: &[u8]) -> Result<Option<TinyAsciiStr<8>>, ParserError> {
+        if *VALUE_LENGTH.start() > bytes.len() || *VALUE_LENGTH.end() < bytes.len() {
+            return Err(ParserError::InvalidExtension);
+        };
+        match TinyAsciiStr::from_bytes(bytes) {
+            Ok(TRUE_VALUE) => Ok(None),
+            Ok(val) if val.is_ascii_alphanumeric() => Ok(Some(val)),
+            _ => Err(ParserError::InvalidExtension),
+        }
+    }
+
+    pub(crate) fn parse_subtag(t: &[u8]) -> Result<Option<TinyAsciiStr<8>>, ParserError> {
+        let s = TinyAsciiStr::from_bytes(t).map_err(|_| ParserError::InvalidSubtag)?;
         if !VALUE_LENGTH.contains(&t.len()) || !s.is_ascii_alphanumeric() {
             return Err(ParserError::InvalidExtension);
         }
@@ -91,6 +134,13 @@ impl Value {
             Ok(Some(s))
         }
     }
+
+    pub(crate) fn for_each_subtag_str<E, F>(&self, f: &mut F) -> Result<(), E>
+    where
+        F: FnMut(&str) -> Result<(), E>,
+    {
+        self.0.as_slice().iter().map(|t| t.as_str()).try_for_each(f)
+    }
 }
 
 impl FromStr for Value {
