@@ -2,6 +2,60 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
+use alloc::vec;
+use alloc::vec::Vec;
+
+/// Internal: A vector that supports no-allocation, constant values if length 0 or 1.
+#[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
+pub(crate) enum ShortVec<T> {
+    Empty,
+    Single(T),
+    Multi(Vec<T>),
+}
+
+impl<T> ShortVec<T> {
+    #[inline]
+    pub const fn new() -> Self {
+        Self::Empty
+    }
+
+    #[inline]
+    pub const fn new_single(item: T) -> Self {
+        Self::Single(item)
+    }
+
+    pub fn push(&mut self, item: T) {
+        *self = match core::mem::replace(self, Self::Empty) {
+            ShortVec::Empty => ShortVec::Single(item),
+            ShortVec::Single(prev_item) => ShortVec::Multi(vec![prev_item, item]),
+            ShortVec::Multi(mut items) => {
+                items.push(item);
+                ShortVec::Multi(items)
+            }
+        };
+    }
+
+    #[inline]
+    pub fn as_slice(&self) -> &[T] {
+        match self {
+            ShortVec::Empty => &[],
+            ShortVec::Single(v) => core::slice::from_ref(v),
+            ShortVec::Multi(v) => v.as_slice(),
+        }
+    }
+}
+
+impl<T> From<Vec<T>> for ShortVec<T> {
+    fn from(v: Vec<T>) -> Self {
+        match v.len() {
+            0 => ShortVec::Empty,
+            #[allow(clippy::unwrap_used)] // we know that the vec is not empty
+            1 => ShortVec::Single(v.into_iter().next().unwrap()),
+            _ => ShortVec::Multi(v),
+        }
+    }
+}
+
 macro_rules! impl_writeable_for_single_subtag {
     ($type:tt, $sample:literal) => {
         impl core::fmt::Display for $type {
@@ -27,8 +81,8 @@ macro_rules! impl_writeable_for_single_subtag {
     };
 }
 
-macro_rules! impl_writeable_for_subtag_list {
-    ($type:tt, $sample1:literal, $sample2:literal) => {
+macro_rules! impl_writeable_for_each_subtag_str_no_test {
+    ($type:tt) => {
         impl core::fmt::Display for $type {
             fn fmt(&self, f: &mut core::fmt::Formatter) -> core::fmt::Result {
                 writeable::Writeable::write_to(self, f)
@@ -38,29 +92,39 @@ macro_rules! impl_writeable_for_subtag_list {
         impl writeable::Writeable for $type {
             fn write_to<W: core::fmt::Write + ?Sized>(&self, sink: &mut W) -> core::fmt::Result {
                 let mut initial = true;
-                for subtag in self.iter() {
+                self.for_each_subtag_str(&mut |subtag| {
                     if initial {
                         initial = false;
                     } else {
                         sink.write_char('-')?;
                     }
-                    writeable::Writeable::write_to(subtag, sink)?;
-                }
-                Ok(())
+                    sink.write_str(subtag)
+                })
             }
 
             #[inline]
             fn write_len(&self) -> writeable::LengthHint {
-                if self.0.is_none() {
-                    writeable::LengthHint::exact(0)
-                } else {
-                    self.iter()
-                        .map(writeable::Writeable::write_len)
-                        .sum::<writeable::LengthHint>()
-                        + (self.len() - 1)
-                }
+                let mut result = writeable::LengthHint::exact(0);
+                let mut initial = true;
+                self.for_each_subtag_str::<core::convert::Infallible, _>(&mut |subtag| {
+                    if initial {
+                        initial = false;
+                    } else {
+                        result += 1;
+                    }
+                    result += subtag.len();
+                    Ok(())
+                })
+                .expect("infallible");
+                result
             }
         }
+    };
+}
+
+macro_rules! impl_writeable_for_subtag_list {
+    ($type:tt, $sample1:literal, $sample2:literal) => {
+        impl_writeable_for_each_subtag_str_no_test!($type);
 
         #[test]
         fn test_writeable() {
@@ -82,42 +146,7 @@ macro_rules! impl_writeable_for_subtag_list {
 
 macro_rules! impl_writeable_for_tinystr_list {
     ($type:tt, $if_empty:literal, $sample1:literal, $sample2:literal) => {
-        impl core::fmt::Display for $type {
-            fn fmt(&self, f: &mut core::fmt::Formatter) -> core::fmt::Result {
-                writeable::Writeable::write_to(self, f)
-            }
-        }
-
-        impl writeable::Writeable for $type {
-            fn write_to<W: core::fmt::Write + ?Sized>(&self, sink: &mut W) -> core::fmt::Result {
-                let mut initial = true;
-                if self.0.len() == 0 {
-                    sink.write_str($if_empty)?;
-                }
-                for subtag in self.0.iter() {
-                    if initial {
-                        initial = false;
-                    } else {
-                        sink.write_char('-')?;
-                    }
-                    sink.write_str(subtag.as_str())?;
-                }
-                Ok(())
-            }
-
-            #[inline]
-            fn write_len(&self) -> writeable::LengthHint {
-                if self.0.len() == 0 {
-                    writeable::LengthHint::exact($if_empty.len())
-                } else {
-                    self.0
-                        .iter()
-                        .map(|s| s.len())
-                        .sum::<writeable::LengthHint>()
-                        + (self.0.len() - 1)
-                }
-            }
-        }
+        impl_writeable_for_each_subtag_str_no_test!($type);
 
         #[test]
         fn test_writeable() {
@@ -138,63 +167,17 @@ macro_rules! impl_writeable_for_tinystr_list {
 
 macro_rules! impl_writeable_for_key_value {
     ($type:tt, $key1:literal, $value1:literal, $key2:literal, $expected2:literal) => {
-        impl core::fmt::Display for $type {
-            fn fmt(&self, f: &mut core::fmt::Formatter) -> core::fmt::Result {
-                writeable::Writeable::write_to(self, f)
-            }
-        }
-
-        impl writeable::Writeable for $type {
-            fn write_to<W: core::fmt::Write + ?Sized>(&self, sink: &mut W) -> core::fmt::Result {
-                let mut initial = true;
-                for (key, value) in self.iter() {
-                    if initial {
-                        initial = false;
-                    } else {
-                        sink.write_char('-')?;
-                    }
-                    writeable::Writeable::write_to(key, sink)?;
-                    if !writeable::Writeable::write_len(value).is_zero() {
-                        sink.write_char('-')?;
-                        writeable::Writeable::write_to(value, sink)?;
-                    }
-                }
-                Ok(())
-            }
-
-            #[inline]
-            fn write_len(&self) -> writeable::LengthHint {
-                if self.0.is_none() {
-                    writeable::LengthHint::exact(0)
-                } else {
-                    self.iter()
-                        .map(|(key, value)| {
-                            writeable::Writeable::write_len(key)
-                                + writeable::Writeable::write_len(value)
-                                + if writeable::Writeable::write_len(value).is_zero() {
-                                    0
-                                } else {
-                                    1
-                                }
-                        })
-                        .sum::<writeable::LengthHint>()
-                        + (self.len() - 1)
-                }
-            }
-        }
+        impl_writeable_for_each_subtag_str_no_test!($type);
 
         #[test]
         fn test_writeable() {
             writeable::assert_writeable_eq!(&$type::default(), "");
             writeable::assert_writeable_eq!(
-                &$type::from_vec_unchecked(vec![(
-                    $key1.parse().unwrap(),
-                    $value1.parse().unwrap()
-                )]),
+                &$type::from_tuple_vec(vec![($key1.parse().unwrap(), $value1.parse().unwrap())]),
                 core::concat!($key1, "-", $value1),
             );
             writeable::assert_writeable_eq!(
-                &$type::from_vec_unchecked(vec![
+                &$type::from_tuple_vec(vec![
                     ($key1.parse().unwrap(), $value1.parse().unwrap()),
                     ($key2.parse().unwrap(), "true".parse().unwrap())
                 ]),
