@@ -2,9 +2,13 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
+use core::cmp::Ordering;
 use core::str::FromStr;
 
-use crate::parser::{get_subtag_iterator, parse_language_identifier, ParserError, ParserMode};
+use crate::parser::{
+    get_subtag_iterator, parse_language_identifier, parse_language_identifier_without_variants,
+    ParserError, ParserMode,
+};
 use crate::subtags;
 use alloc::string::String;
 use alloc::string::ToString;
@@ -56,6 +60,7 @@ use alloc::string::ToString;
 ///
 /// [`Unicode BCP47 Language Identifier`]: https://unicode.org/reports/tr35/tr35.html#Unicode_language_identifier
 #[derive(Default, PartialEq, Eq, Clone, Hash, PartialOrd, Ord)]
+#[allow(clippy::exhaustive_structs)] // This struct is stable (and invoked by a macro)
 pub struct LanguageIdentifier {
     /// Language subtag of the language identifier.
     pub language: subtags::Language,
@@ -85,6 +90,22 @@ impl LanguageIdentifier {
         parse_language_identifier(v, ParserMode::LanguageIdentifier)
     }
 
+    #[doc(hidden)]
+    // The return type should be `Result<Self, ParserError>` once the `const_precise_live_drops`
+    // is stabilized ([rust-lang#73255](https://github.com/rust-lang/rust/issues/73255)).
+    pub const fn from_bytes_without_variants(
+        v: &[u8],
+    ) -> Result<
+        (
+            subtags::Language,
+            Option<subtags::Script>,
+            Option<subtags::Region>,
+        ),
+        ParserError,
+    > {
+        parse_language_identifier_without_variants(v, ParserMode::LanguageIdentifier)
+    }
+
     /// A constructor which takes a utf8 slice which may contain extension keys,
     /// parses it and produces a well-formed [`LanguageIdentifier`].
     ///
@@ -105,26 +126,22 @@ impl LanguageIdentifier {
         parse_language_identifier(v, ParserMode::Locale)
     }
 
-    /// Returns the default undefined language "und". Same as [`default()`](Default::default()), but is `const`.
+    /// The default undefined language "und". Same as [`default()`](Default::default()).
     ///
     /// # Examples
     ///
     /// ```
     /// use icu::locid::LanguageIdentifier;
     ///
-    /// const langid: LanguageIdentifier = LanguageIdentifier::und();
-    /// assert_eq!(LanguageIdentifier::default(), langid);
-    /// assert_eq!("und", langid.to_string());
+    /// assert_eq!(LanguageIdentifier::default(), LanguageIdentifier::UND);
+    /// assert_eq!("und", LanguageIdentifier::UND.to_string());
     /// ```
-    #[inline]
-    pub const fn und() -> Self {
-        Self {
-            language: subtags::Language::und(),
-            script: None,
-            region: None,
-            variants: subtags::Variants::new(),
-        }
-    }
+    pub const UND: Self = Self {
+        language: subtags::Language::UND,
+        script: None,
+        region: None,
+        variants: subtags::Variants::new(),
+    };
 
     /// This is a best-effort operation that performs all available levels of canonicalization.
     ///
@@ -142,6 +159,76 @@ impl LanguageIdentifier {
         let lang_id = Self::from_bytes(input.as_ref())?;
         Ok(lang_id.to_string())
     }
+
+    /// Compare this `LanguageIdentifier` with a BCP-47 string.
+    ///
+    /// The return value is equivalent to what would happen if you first converted this
+    /// `LanguageIdentifier` to a BCP-47 string and then performed a byte comparison.
+    ///
+    /// This function is case-sensitive and results in a *total order*, so it is appropriate for
+    /// binary search. The only argument producing [`Ordering::Equal`] is `self.to_string()`.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use icu::locid::LanguageIdentifier;
+    /// use std::cmp::Ordering;
+    ///
+    /// let bcp47_strings: &[&[u8]] = &[
+    ///     b"pl-Latn-PL",
+    ///     b"und",
+    ///     b"und-Adlm",
+    ///     b"und-GB",
+    ///     b"und-ZA",
+    ///     b"und-fonipa",
+    ///     b"zh",
+    /// ];
+    ///
+    /// for ab in bcp47_strings.windows(2) {
+    ///     let a = ab[0];
+    ///     let b = ab[1];
+    ///     assert!(a.cmp(b) == Ordering::Less);
+    ///     let a_langid = LanguageIdentifier::from_bytes(a).unwrap();
+    ///     assert!(a_langid.cmp_bytes(b) == Ordering::Less);
+    /// }
+    /// ```
+    pub fn cmp_bytes(&self, other: &[u8]) -> Ordering {
+        let mut other_iter = other.split(|b| *b == b'-');
+        let r = self.for_each_subtag_str(&mut |subtag| {
+            if let Some(other) = other_iter.next() {
+                match subtag.as_bytes().cmp(other) {
+                    Ordering::Equal => Ok(()),
+                    not_equal => Err(not_equal),
+                }
+            } else {
+                Err(Ordering::Greater)
+            }
+        });
+        if let Err(o) = r {
+            return o;
+        }
+        if other_iter.next().is_some() {
+            return Ordering::Less;
+        }
+        Ordering::Equal
+    }
+
+    pub(crate) fn for_each_subtag_str<E, F>(&self, f: &mut F) -> Result<(), E>
+    where
+        F: FnMut(&str) -> Result<(), E>,
+    {
+        f(self.language.as_str())?;
+        if let Some(ref script) = self.script {
+            f(script.as_str())?;
+        }
+        if let Some(ref region) = self.region {
+            f(region.as_str())?;
+        }
+        for variant in self.variants.iter() {
+            f(variant.as_str())?;
+        }
+        Ok(())
+    }
 }
 
 impl AsRef<LanguageIdentifier> for LanguageIdentifier {
@@ -170,49 +257,12 @@ impl FromStr for LanguageIdentifier {
     }
 }
 
-impl core::fmt::Display for LanguageIdentifier {
-    fn fmt(&self, f: &mut core::fmt::Formatter) -> core::fmt::Result {
-        writeable::Writeable::write_to(self, f)
-    }
-}
-
-impl writeable::Writeable for LanguageIdentifier {
-    fn write_to<W: core::fmt::Write + ?Sized>(&self, sink: &mut W) -> core::fmt::Result {
-        writeable::Writeable::write_to(&self.language, sink)?;
-        if let Some(ref script) = self.script {
-            sink.write_char('-')?;
-            writeable::Writeable::write_to(script, sink)?;
-        }
-        if let Some(ref region) = self.region {
-            sink.write_char('-')?;
-            writeable::Writeable::write_to(region, sink)?;
-        }
-        if !self.variants.is_empty() {
-            sink.write_char('-')?;
-            writeable::Writeable::write_to(&self.variants, sink)?;
-        }
-        Ok(())
-    }
-
-    fn write_len(&self) -> writeable::LengthHint {
-        let mut result = writeable::Writeable::write_len(&self.language);
-        if let Some(ref script) = self.script {
-            result += writeable::Writeable::write_len(script) + 1;
-        }
-        if let Some(ref region) = self.region {
-            result += writeable::Writeable::write_len(region) + 1;
-        }
-        if !self.variants.is_empty() {
-            result += writeable::Writeable::write_len(&self.variants) + 1;
-        }
-        result
-    }
-}
+impl_writeable_for_each_subtag_str_no_test!(LanguageIdentifier);
 
 #[test]
 fn test_writeable() {
     use writeable::assert_writeable_eq;
-    assert_writeable_eq!(LanguageIdentifier::und(), "und");
+    assert_writeable_eq!(LanguageIdentifier::UND, "und");
     assert_writeable_eq!(LanguageIdentifier::from_str("und-001").unwrap(), "und-001");
     assert_writeable_eq!(
         LanguageIdentifier::from_str("und-Mymr").unwrap(),
@@ -271,3 +321,106 @@ impl PartialEq<str> for LanguageIdentifier {
         iter.next() == None
     }
 }
+
+/// # Examples
+///
+/// ```
+/// use icu::locid::LanguageIdentifier;
+/// use icu::locid::language;
+///
+/// let language = language!("en");
+/// let li = LanguageIdentifier::from(language);
+///
+/// assert_eq!(li.language, "en");
+/// assert_eq!(li, "en");
+/// ```
+impl From<subtags::Language> for LanguageIdentifier {
+    fn from(language: subtags::Language) -> Self {
+        Self {
+            language,
+            ..Default::default()
+        }
+    }
+}
+
+/// # Examples
+///
+/// ```
+/// use icu::locid::LanguageIdentifier;
+/// use icu::locid::script;
+///
+/// let script = script!("latn");
+/// let li = LanguageIdentifier::from(Some(script));
+///
+/// assert_eq!(li.script.unwrap(), "Latn");
+/// assert_eq!(li, "und-Latn");
+/// ```
+impl From<Option<subtags::Script>> for LanguageIdentifier {
+    fn from(script: Option<subtags::Script>) -> Self {
+        Self {
+            script,
+            ..Default::default()
+        }
+    }
+}
+
+/// # Examples
+///
+/// ```
+/// use icu::locid::LanguageIdentifier;
+/// use icu::locid::region;
+///
+/// let region = region!("US");
+/// let li = LanguageIdentifier::from(Some(region));
+///
+/// assert_eq!(li.region.unwrap(), "US");
+/// assert_eq!(li, "und-US");
+/// ```
+impl From<Option<subtags::Region>> for LanguageIdentifier {
+    fn from(region: Option<subtags::Region>) -> Self {
+        Self {
+            region,
+            ..Default::default()
+        }
+    }
+}
+
+/// # Examples
+///
+/// ```
+/// use icu::locid::LanguageIdentifier;
+/// use icu::locid::{language, script, region};
+///
+/// let lang = language!("en");
+/// let script = script!("Latn");
+/// let region = region!("US");
+/// let li = LanguageIdentifier::from((lang, Some(script), Some(region)));
+///
+/// assert_eq!(li.language, "en");
+/// assert_eq!(li.script.unwrap(), "Latn");
+/// assert_eq!(li.region.unwrap(), "US");
+/// assert_eq!(li.variants.len(), 0);
+/// assert_eq!(li, "en-Latn-US");
+/// ```
+impl
+    From<(
+        subtags::Language,
+        Option<subtags::Script>,
+        Option<subtags::Region>,
+    )> for LanguageIdentifier
+{
+    fn from(
+        lsr: (
+            subtags::Language,
+            Option<subtags::Script>,
+            Option<subtags::Region>,
+        ),
+    ) -> Self {
+        Self {
+            language: lsr.0,
+            script: lsr.1,
+            region: lsr.2,
+            ..Default::default()
+        }
+    }
+}
