@@ -43,6 +43,18 @@
 //!
 //! assert_eq!(loc.to_string(), "en-GB");
 //! ```
+//!
+//! ## Macros
+//!
+//! ```rust
+//! use icu::locid::{language, region, langid};
+//!
+//! let lid = langid!("EN_US");
+//!
+//! assert_eq!(lid.language, language!("en"));
+//! assert_eq!(lid.region, Some(region!("US")));
+//! ```
+
 //!
 //! For more details, see [`Locale`] and [`LanguageIdentifier`].
 //!
@@ -50,8 +62,20 @@
 //! [`ICU4X`]: ../icu/index.html
 //! [`Unicode Extensions`]: extensions
 
+// https://github.com/unicode-org/icu4x/blob/main/docs/process/boilerplate.md#library-annotations
+#![cfg_attr(not(any(test, feature = "std")), no_std)]
+#![cfg_attr(
+    not(test),
+    deny(
+        clippy::indexing_slicing,
+        clippy::unwrap_used,
+        clippy::expect_used,
+        clippy::panic,
+        clippy::exhaustive_structs,
+        clippy::exhaustive_enums
+    )
+)]
 #![warn(missing_docs)]
-#![cfg_attr(not(any(feature = "std", test)), no_std)]
 
 extern crate alloc;
 
@@ -61,10 +85,13 @@ mod helpers;
 pub mod extensions;
 mod langid;
 mod locale;
+mod macros;
 mod parser;
 #[cfg(feature = "serde")]
 mod serde;
 pub mod subtags;
+#[cfg(feature = "zerovec")]
+pub mod zerovec;
 
 pub use langid::LanguageIdentifier;
 pub use locale::Locale;
