@@ -6,6 +6,7 @@ use crate::parser::{get_subtag_iterator, parse_locale, ParserError};
 use crate::{extensions, subtags, LanguageIdentifier};
 use alloc::string::String;
 use alloc::string::ToString;
+use core::cmp::Ordering;
 use core::str::FromStr;
 
 /// A core struct representing a [`Unicode Locale Identifier`].
@@ -35,7 +36,7 @@ use core::str::FromStr;
 ///
 /// let key: Key = "ca".parse().expect("Parsing key failed.");
 /// let value: Value = "buddhist".parse().expect("Parsing value failed.");
-/// assert_eq!(loc.extensions.unicode.keywords.get(key),
+/// assert_eq!(loc.extensions.unicode.keywords.get(&key),
 ///     Some(&value));
 /// ```
 ///
@@ -51,7 +52,7 @@ use core::str::FromStr;
 /// `_` separators to `-` and adjusting casing to conform to the Unicode standard.
 ///
 /// Any bogus subtags will cause the parsing to fail with an error.
-/// No subtag validation is performed.
+/// No subtag validation or canonicalization is performed.
 ///
 /// # Examples
 ///
@@ -69,6 +70,7 @@ use core::str::FromStr;
 /// [`Unicode Locale Identifier`]: https://unicode.org/reports/tr35/tr35.html#Unicode_locale_identifier
 #[derive(Default, PartialEq, Eq, Clone, Hash, PartialOrd, Ord)]
 #[allow(missing_docs)] // TODO(#1028) - Add missing docs.
+#[allow(clippy::exhaustive_structs)] // This struct is stable (and invoked by a macro)
 pub struct Locale {
     // Language component of the Locale
     pub id: LanguageIdentifier,
@@ -94,24 +96,20 @@ impl Locale {
         parse_locale(v)
     }
 
-    /// Returns the default undefined locale "und". Same as [`default()`](Default::default()), but is `const`.
+    /// The default undefined locale "und". Same as [`default()`](Default::default()).
     ///
     /// # Examples
     ///
     /// ```
     /// use icu::locid::Locale;
     ///
-    /// const loc: Locale = Locale::und();
-    /// assert_eq!(Locale::default(), loc);
-    /// assert_eq!("und", loc.to_string());
+    /// assert_eq!(Locale::default(), Locale::UND);
+    /// assert_eq!("und", Locale::UND.to_string());
     /// ```
-    #[inline]
-    pub const fn und() -> Self {
-        Self {
-            id: LanguageIdentifier::und(),
-            extensions: extensions::Extensions::new(),
-        }
-    }
+    pub const UND: Self = Self {
+        id: LanguageIdentifier::UND,
+        extensions: extensions::Extensions::new(),
+    };
 
     /// This is a best-effort operation that performs all available levels of canonicalization.
     ///
@@ -130,26 +128,66 @@ impl Locale {
         Ok(locale.to_string())
     }
 
-    /// Accessor method for unicode extensions.
+    /// Compare this `Locale` with a BCP-47 string.
+    ///
+    /// The return value is equivalent to what would happen if you first converted this
+    /// `Locale` to a BCP-47 string and then performed a byte comparison.
+    ///
+    /// This function is case-sensitive and results in a *total order*, so it is appropriate for
+    /// binary search. The only argument producing [`Ordering::Equal`] is `self.to_string()`.
     ///
     /// # Examples
     ///
     /// ```
-    /// use icu::locid::{extensions::unicode::Key, Locale};
+    /// use icu::locid::Locale;
+    /// use std::cmp::Ordering;
     ///
-    /// let loc = Locale::from_bytes("en-US-u-hc-h12".as_bytes())
-    ///     .expect("Parsing failed.");
-    /// let key = "hc".parse::<Key>()
-    ///     .expect("Invalid key.");
-    /// let ext = loc.get_unicode_extension(&key)
-    ///     .expect("Extension not defined");
-    /// assert_eq!(ext.to_string(), "h12");
+    /// let bcp47_strings: &[&[u8]] = &[
+    ///     b"pl-Latn-PL",
+    ///     b"und",
+    ///     b"und-fonipa",
+    ///     b"und-t-m0-true",
+    ///     b"und-u-ca-hebrew",
+    ///     b"und-u-ca-japanese",
+    ///     b"zh",
+    /// ];
+    ///
+    /// for ab in bcp47_strings.windows(2) {
+    ///     let a = ab[0];
+    ///     let b = ab[1];
+    ///     assert!(a.cmp(b) == Ordering::Less);
+    ///     let a_langid = Locale::from_bytes(a).unwrap();
+    ///     assert!(a_langid.cmp_bytes(b) == Ordering::Less);
+    /// }
     /// ```
-    pub fn get_unicode_extension(
-        &self,
-        key: &extensions::unicode::Key,
-    ) -> Option<&extensions::unicode::Value> {
-        self.extensions.unicode.keywords.get(key)
+    pub fn cmp_bytes(&self, other: &[u8]) -> Ordering {
+        let mut other_iter = other.split(|b| *b == b'-');
+        let r = self.for_each_subtag_str(&mut |subtag| {
+            if let Some(other) = other_iter.next() {
+                match subtag.as_bytes().cmp(other) {
+                    Ordering::Equal => Ok(()),
+                    not_equal => Err(not_equal),
+                }
+            } else {
+                Err(Ordering::Greater)
+            }
+        });
+        if let Err(o) = r {
+            return o;
+        }
+        if other_iter.next().is_some() {
+            return Ordering::Less;
+        }
+        Ordering::Equal
+    }
+
+    pub(crate) fn for_each_subtag_str<E, F>(&self, f: &mut F) -> Result<(), E>
+    where
+        F: FnMut(&str) -> Result<(), E>,
+    {
+        self.id.for_each_subtag_str(f)?;
+        self.extensions.for_each_subtag_str(f)?;
+        Ok(())
     }
 }
 
@@ -194,30 +232,12 @@ impl core::fmt::Debug for Locale {
     }
 }
 
-impl core::fmt::Display for Locale {
-    fn fmt(&self, f: &mut core::fmt::Formatter) -> core::fmt::Result {
-        writeable::Writeable::write_to(self, f)
-    }
-}
-
-impl writeable::Writeable for Locale {
-    fn write_to<W: core::fmt::Write + ?Sized>(&self, sink: &mut W) -> core::fmt::Result {
-        writeable::Writeable::write_to(&self.id, sink)?;
-        writeable::Writeable::write_to(&self.extensions, sink)?;
-        Ok(())
-    }
-
-    fn write_len(&self) -> writeable::LengthHint {
-        let mut result = writeable::Writeable::write_len(&self.id);
-        result += writeable::Writeable::write_len(&self.extensions);
-        result
-    }
-}
+impl_writeable_for_each_subtag_str_no_test!(Locale);
 
 #[test]
 fn test_writeable() {
     use writeable::assert_writeable_eq;
-    assert_writeable_eq!(Locale::und(), "und");
+    assert_writeable_eq!(Locale::UND, "und");
     assert_writeable_eq!(Locale::from_str("und-001").unwrap(), "und-001");
     assert_writeable_eq!(Locale::from_str("und-Mymr").unwrap(), "und-Mymr");
     assert_writeable_eq!(Locale::from_str("my-Mymr-MM").unwrap(), "my-Mymr-MM");
@@ -241,6 +261,7 @@ fn test_writeable() {
         Locale::from_str("en-001-x-foo-bar").unwrap(),
         "en-001-x-foo-bar",
     );
+    assert_writeable_eq!(Locale::from_str("und-t-m0-true").unwrap(), "und-t-m0-true",);
 }
 
 impl PartialEq<&str> for Locale {
@@ -260,7 +281,7 @@ macro_rules! subtag_matches {
 
 impl PartialEq<str> for Locale {
     fn eq(&self, other: &str) -> bool {
-        let mut iter = get_subtag_iterator(other.as_bytes()).peekable();
+        let mut iter = get_subtag_iterator(other.as_bytes());
         if !subtag_matches!(subtags::Language, iter, self.id.language) {
             return false;
         }
@@ -294,3 +315,104 @@ impl PartialEq<str> for Locale {
         iter.next() == None
     }
 }
+
+/// # Examples
+///
+/// ```
+/// use icu::locid::Locale;
+/// use icu::locid::language;
+///
+/// let language = language!("en");
+/// let loc = Locale::from(language);
+///
+/// assert_eq!(loc.id.language, "en");
+/// assert_eq!(loc, "en");
+/// ```
+impl From<subtags::Language> for Locale {
+    fn from(language: subtags::Language) -> Self {
+        Self {
+            id: language.into(),
+            ..Default::default()
+        }
+    }
+}
+
+/// # Examples
+///
+/// ```
+/// use icu::locid::Locale;
+/// use icu::locid::script;
+///
+/// let script = script!("latn");
+/// let loc = Locale::from(Some(script));
+///
+/// assert_eq!(loc.id.script.unwrap(), "Latn");
+/// assert_eq!(loc, "und-Latn");
+/// ```
+impl From<Option<subtags::Script>> for Locale {
+    fn from(script: Option<subtags::Script>) -> Self {
+        Self {
+            id: script.into(),
+            ..Default::default()
+        }
+    }
+}
+
+/// # Examples
+///
+/// ```
+/// use icu::locid::Locale;
+/// use icu::locid::region;
+///
+/// let region = region!("US");
+/// let loc = Locale::from(Some(region));
+///
+/// assert_eq!(loc.id.region.unwrap(), "US");
+/// assert_eq!(loc, "und-US");
+/// ```
+impl From<Option<subtags::Region>> for Locale {
+    fn from(region: Option<subtags::Region>) -> Self {
+        Self {
+            id: region.into(),
+            ..Default::default()
+        }
+    }
+}
+
+/// # Examples
+///
+/// ```
+/// use icu::locid::Locale;
+/// use icu::locid::{language, script, region};
+///
+/// let lang = language!("en");
+/// let script = script!("Latn");
+/// let region = region!("US");
+/// let loc = Locale::from((lang, Some(script), Some(region)));
+///
+/// assert_eq!(loc.id.language, "en");
+/// assert_eq!(loc.id.script.unwrap(), "Latn");
+/// assert_eq!(loc.id.region.unwrap(), "US");
+/// assert_eq!(loc.id.variants.len(), 0);
+/// assert_eq!(loc, "en-Latn-US");
+/// ```
+impl
+    From<(
+        subtags::Language,
+        Option<subtags::Script>,
+        Option<subtags::Region>,
+    )> for Locale
+{
+    fn from(
+        lsr: (
+            subtags::Language,
+            Option<subtags::Script>,
+            Option<subtags::Region>,
+        ),
+    ) -> Self {
+        Self {
+            id: lsr.into(),
+            ..Default::default()
+        }
+    }
+}
