@@ -8,7 +8,7 @@ use crate::parser::{get_subtag_iterator, parse_language_identifier_from_iter, Pa
 use crate::Locale;
 
 pub fn parse_locale(t: &[u8]) -> Result<Locale, ParserError> {
-    let mut iter = get_subtag_iterator(t).peekable();
+    let mut iter = get_subtag_iterator(t);
 
     let id = parse_language_identifier_from_iter(&mut iter, ParserMode::Locale)?;
     let extensions = if iter.peek().is_some() {
