@@ -6,7 +6,7 @@ use crate::parser::errors::ParserError;
 use core::fmt;
 use core::ops::RangeInclusive;
 use core::str::FromStr;
-use tinystr::{tinystr4, TinyStr4};
+use tinystr::TinyAsciiStr;
 
 /// A language subtag (examples: `"en"`, `"csb"`, `"zh"`, `"und"`, etc.)
 ///
@@ -38,11 +38,12 @@ use tinystr::{tinystr4, TinyStr4};
 /// but that form has not been used and ICU4X does not support it right now.
 ///
 /// [`unicode_language_id`]: https://unicode.org/reports/tr35/#unicode_language_id
-#[derive(Default, Debug, PartialEq, Eq, Clone, Hash, PartialOrd, Ord, Copy)]
-pub struct Language(Option<TinyStr4>);
+#[derive(Debug, PartialEq, Eq, Clone, Hash, PartialOrd, Ord, Copy)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize))]
+#[repr(transparent)]
+pub struct Language(TinyAsciiStr<{ *LANGUAGE_LENGTH.end() }>);
 
 const LANGUAGE_LENGTH: RangeInclusive<usize> = 2..=3;
-const UND_VALUE: TinyStr4 = tinystr4!("und");
 
 impl Language {
     /// A constructor which takes a utf8 slice, parses it and
@@ -58,25 +59,56 @@ impl Language {
     ///
     /// assert_eq!(lang, "en");
     /// ```
-    pub fn from_bytes(v: &[u8]) -> Result<Self, ParserError> {
-        let slen = v.len();
+    pub const fn from_bytes(v: &[u8]) -> Result<Self, ParserError> {
+        Self::from_bytes_manual_slice(v, 0, v.len())
+    }
 
-        if !LANGUAGE_LENGTH.contains(&slen) {
+    /// Equivalent to [`from_bytes(bytes[start..end])`](Self::from_bytes),
+    /// but callable in a `const` context (which range indexing is not).
+    pub const fn from_bytes_manual_slice(
+        v: &[u8],
+        start: usize,
+        end: usize,
+    ) -> Result<Self, ParserError> {
+        let slen = end - start;
+
+        if slen < *LANGUAGE_LENGTH.start() || *LANGUAGE_LENGTH.end() < slen {
             return Err(ParserError::InvalidLanguage);
         }
 
-        let s = TinyStr4::from_bytes(v).map_err(|_| ParserError::InvalidLanguage)?;
-
-        if !s.is_ascii_alphabetic() {
-            return Err(ParserError::InvalidLanguage);
+        match TinyAsciiStr::from_bytes_manual_slice(v, start, end) {
+            Ok(s) if s.is_ascii_alphabetic() => Ok(Self(s.to_ascii_lowercase())),
+            _ => Err(ParserError::InvalidLanguage),
         }
+    }
 
-        let value = s.to_ascii_lowercase();
-
-        if value == UND_VALUE {
-            Ok(Self(None))
+    /// Safely creates a [`Language`] from a reference to its raw format
+    /// as returned by [`Language::into_raw()`].
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use icu::locid::subtags::Language;
+    ///
+    /// assert!(matches!(Language::try_from_raw(*b"de\0"), Ok(_)));
+    /// assert!(matches!(Language::try_from_raw(*b"ars"), Ok(_)));
+    /// assert!(matches!(Language::try_from_raw(*b"419"), Err(_)));
+    ///
+    /// // Unlike the other constructors, this one is case-sensitive:
+    /// assert!(matches!(Language::try_from_raw(*b"EN\0"), Err(_)));
+    /// ```
+    pub fn try_from_raw(v: [u8; 3]) -> Result<Self, ParserError> {
+        let s = TinyAsciiStr::<{ core::mem::size_of::<Self>() }>::try_from_raw(v)
+            .map_err(|_| ParserError::InvalidSubtag)?;
+        let is_valid = match s.len() {
+            // LANGUAGE_LENGTH
+            2..=3 => s.is_ascii_alphabetic_lowercase(),
+            _ => false,
+        };
+        if is_valid {
+            Ok(Self(s))
         } else {
-            Ok(Self(Some(value)))
+            Err(ParserError::InvalidSubtag)
         }
     }
 
@@ -95,8 +127,8 @@ impl Language {
     /// let lang = unsafe { Language::from_raw_unchecked(raw) };
     /// assert_eq!(lang, "en");
     /// ```
-    pub fn into_raw(self) -> Option<u32> {
-        self.0.map(Into::<u32>::into)
+    pub fn into_raw(self) -> [u8; 3] {
+        *self.0.all_bytes()
     }
 
     /// Constructor which takes a raw value returned by
@@ -117,30 +149,23 @@ impl Language {
     ///
     /// # Safety
     ///
-    /// This function accepts a [`u32`] that is expected to be a valid [`TinyStr4`]
+    /// This function accepts a [`[u8; 3]`] that is expected to be a valid [`TinyAsciiStr<3>`]
     /// representing a [`Language`] subtag in canonical syntax.
-    pub const unsafe fn from_raw_unchecked(v: Option<u32>) -> Self {
-        Self(match v {
-            Some(v) => Some(TinyStr4::new_unchecked(v)),
-            None => None,
-        })
+    pub const unsafe fn from_raw_unchecked(v: [u8; 3]) -> Self {
+        Self(TinyAsciiStr::from_bytes_unchecked(v))
     }
 
-    /// Returns the default undefined language "und". Same as [`default()`](Default::default()), but is `const`.
+    /// The default undefined language "und". Same as [`default()`](Default::default()).
     ///
     /// # Examples
     ///
     /// ```
     /// use icu::locid::subtags::Language;
     ///
-    /// const language: Language = Language::und();
-    /// assert_eq!(Language::default(), language);
-    /// assert_eq!("und", language.to_string());
+    /// assert_eq!(Language::default(), Language::UND);
+    /// assert_eq!("und", Language::UND.to_string());
     /// ```
-    #[inline]
-    pub const fn und() -> Self {
-        Self(None)
-    }
+    pub const UND: Self = crate::language!("und");
 
     /// A helper function for displaying
     /// a [`Language`] subtag as a `&`[`str`].
@@ -158,11 +183,12 @@ impl Language {
     ///
     /// `Notice`: For many use cases, such as comparison,
     /// [`Language`] implements [`PartialEq`]`<&`[`str`]`>` which allows for direct comparisons.
+    #[inline]
     pub fn as_str(&self) -> &str {
-        self.0.as_deref().unwrap_or("und")
+        self.0.as_str()
     }
 
-    /// Resets the [`Language`] subtag to an empty one.
+    /// Resets the [`Language`] subtag to an empty one (equal to `"und"`).
     ///
     /// # Examples
     ///
@@ -178,11 +204,12 @@ impl Language {
     ///
     /// assert_eq!(lang.as_str(), "und");
     /// ```
+    #[inline]
     pub fn clear(&mut self) {
-        self.0.take();
+        *self = Self::UND
     }
 
-    /// Tests if the [`Language`] subtag is empty.
+    /// Tests if the [`Language`] subtag is empty (equal to `"und"`).
     ///
     /// # Examples
     ///
@@ -198,8 +225,9 @@ impl Language {
     ///
     /// assert_eq!(lang.is_empty(), true);
     /// ```
+    #[inline]
     pub fn is_empty(self) -> bool {
-        self.0.is_none()
+        self == Self::UND
     }
 }
 
@@ -224,7 +252,7 @@ impl writeable::Writeable for Language {
 
     #[inline]
     fn write_len(&self) -> writeable::LengthHint {
-        writeable::LengthHint::exact(self.0.map_or(3, |t| t.len()))
+        writeable::LengthHint::exact(self.0.len())
     }
 }
 
@@ -247,8 +275,14 @@ impl<'l> From<&'l Language> for &'l str {
     }
 }
 
-impl From<Language> for Option<TinyStr4> {
+impl From<Language> for TinyAsciiStr<3> {
     fn from(input: Language) -> Self {
-        input.0.map(Into::into)
+        input.0
+    }
+}
+
+impl Default for Language {
+    fn default() -> Language {
+        Language::UND
     }
 }
