@@ -4,7 +4,7 @@
 
 use crate::parser::errors::ParserError;
 use core::str::FromStr;
-use tinystr::TinyStr4;
+use tinystr::TinyAsciiStr;
 
 /// A region subtag (examples: `"US"`, `"CN"`, `"AR"` etc.)
 ///
@@ -22,7 +22,9 @@ use tinystr::TinyStr4;
 ///
 /// [`unicode_region_id`]: https://unicode.org/reports/tr35/#unicode_region_id
 #[derive(Debug, PartialEq, Eq, Clone, Hash, PartialOrd, Ord, Copy)]
-pub struct Region(TinyStr4);
+#[cfg_attr(feature = "serde", derive(serde::Serialize))]
+#[repr(transparent)]
+pub struct Region(TinyAsciiStr<REGION_NUM_LENGTH>);
 
 const REGION_ALPHA_LENGTH: usize = 2;
 const REGION_NUM_LENGTH: usize = 3;
@@ -41,26 +43,61 @@ impl Region {
     ///
     /// assert_eq!(region, "FR");
     /// ```
-    pub fn from_bytes(v: &[u8]) -> Result<Self, ParserError> {
-        match v.len() {
-            REGION_ALPHA_LENGTH => {
-                let s = TinyStr4::from_bytes(v).map_err(|_| ParserError::InvalidSubtag)?;
-                if !s.is_ascii_alphabetic() {
-                    return Err(ParserError::InvalidSubtag);
-                }
-                Ok(Self(s.to_ascii_uppercase()))
-            }
-            REGION_NUM_LENGTH => {
-                let s = TinyStr4::from_bytes(v).map_err(|_| ParserError::InvalidSubtag)?;
-                if !s.is_ascii_numeric() {
-                    return Err(ParserError::InvalidSubtag);
-                }
-                Ok(Self(s))
-            }
+    pub const fn from_bytes(v: &[u8]) -> Result<Self, ParserError> {
+        Self::from_bytes_manual_slice(v, 0, v.len())
+    }
+
+    /// Equivalent to [`from_bytes(bytes[start..end])`](Self::from_bytes),
+    /// but callable in a `const` context (which range indexing is not).
+    pub const fn from_bytes_manual_slice(
+        v: &[u8],
+        start: usize,
+        end: usize,
+    ) -> Result<Self, ParserError> {
+        match end - start {
+            REGION_ALPHA_LENGTH => match TinyAsciiStr::from_bytes_manual_slice(v, start, end) {
+                Ok(s) if s.is_ascii_alphabetic() => Ok(Self(s.to_ascii_uppercase())),
+                _ => Err(ParserError::InvalidSubtag),
+            },
+            REGION_NUM_LENGTH => match TinyAsciiStr::from_bytes_manual_slice(v, start, end) {
+                Ok(s) if s.is_ascii_numeric() => Ok(Self(s)),
+                _ => Err(ParserError::InvalidSubtag),
+            },
             _ => Err(ParserError::InvalidSubtag),
         }
     }
 
+    /// Safely creates a [`Region`] from a reference to its raw format
+    /// as returned by [`Region::into_raw()`].
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use icu::locid::subtags::Region;
+    ///
+    /// assert!(matches!(Region::try_from_raw(*b"US\0"), Ok(_)));
+    /// assert!(matches!(Region::try_from_raw(*b"419"), Ok(_)));
+    /// assert!(matches!(Region::try_from_raw(*b"foo"), Err(_)));
+    /// assert!(matches!(Region::try_from_raw(*b"US0"), Err(_)));
+    ///
+    /// // Unlike the other constructors, this one is case-sensitive:
+    /// assert!(matches!(Region::try_from_raw(*b"us\0"), Err(_)));
+    /// ```
+    pub fn try_from_raw(v: [u8; 3]) -> Result<Self, ParserError> {
+        let s = TinyAsciiStr::<{ core::mem::size_of::<Self>() }>::try_from_raw(v)
+            .map_err(|_| ParserError::InvalidSubtag)?;
+        let is_valid = match s.len() {
+            REGION_ALPHA_LENGTH => s.is_ascii_alphabetic_uppercase(),
+            REGION_NUM_LENGTH => s.is_ascii_numeric(),
+            _ => false,
+        };
+        if is_valid {
+            Ok(Self(s))
+        } else {
+            Err(ParserError::InvalidSubtag)
+        }
+    }
+
     /// Deconstructs the [`Region`] into raw format to be consumed
     /// by [`from_raw_unchecked()`](Region::from_raw_unchecked()).
     ///
@@ -76,8 +113,8 @@ impl Region {
     /// let region = unsafe { Region::from_raw_unchecked(raw) };
     /// assert_eq!(region, "US");
     /// ```
-    pub fn into_raw(self) -> u32 {
-        self.0.into()
+    pub fn into_raw(self) -> [u8; 3] {
+        *self.0.all_bytes()
     }
 
     /// Constructor which takes a raw value returned by
@@ -98,10 +135,10 @@ impl Region {
     ///
     /// # Safety
     ///
-    /// This function accepts a [`u32`] that is expected to be a valid [`TinyStr4`]
+    /// This function accepts a [`[u8; 3]`] that is expected to be a valid [`TinyAsciiStr<3>`]
     /// representing a [`Region`] subtag in canonical syntax.
-    pub const unsafe fn from_raw_unchecked(v: u32) -> Self {
-        Self(TinyStr4::new_unchecked(v))
+    pub const unsafe fn from_raw_unchecked(v: [u8; 3]) -> Self {
+        Self(TinyAsciiStr::from_bytes_unchecked(v))
     }
 
     /// A helper function for displaying
@@ -163,7 +200,7 @@ impl<'l> From<&'l Region> for &'l str {
     }
 }
 
-impl From<Region> for TinyStr4 {
+impl From<Region> for TinyAsciiStr<3> {
     fn from(input: Region) -> Self {
         input.0
     }
