@@ -4,7 +4,7 @@
 
 use crate::parser::errors::ParserError;
 use core::str::FromStr;
-use tinystr::TinyStr4;
+use tinystr::TinyAsciiStr;
 
 /// A script subtag (examples: `"Latn"`, `"Arab"`, etc.)
 ///
@@ -22,7 +22,9 @@ use tinystr::TinyStr4;
 ///
 /// [`unicode_script_id`]: https://unicode.org/reports/tr35/#unicode_script_id
 #[derive(Debug, PartialEq, Eq, Clone, Hash, PartialOrd, Ord, Copy)]
-pub struct Script(TinyStr4);
+#[cfg_attr(feature = "serde", derive(serde::Serialize))]
+#[repr(transparent)]
+pub struct Script(TinyAsciiStr<SCRIPT_LENGTH>);
 
 pub const SCRIPT_LENGTH: usize = 4;
 
@@ -40,16 +42,54 @@ impl Script {
     ///
     /// assert_eq!(script, "Latn");
     /// ```
-    pub fn from_bytes(v: &[u8]) -> Result<Self, ParserError> {
-        if v.len() != SCRIPT_LENGTH {
+    pub const fn from_bytes(v: &[u8]) -> Result<Self, ParserError> {
+        Self::from_bytes_manual_slice(v, 0, v.len())
+    }
+
+    /// Equivalent to [`from_bytes(bytes[start..end])`](Self::from_bytes),
+    /// but callable in a `const` context (which range indexing is not).
+    pub const fn from_bytes_manual_slice(
+        v: &[u8],
+        start: usize,
+        end: usize,
+    ) -> Result<Self, ParserError> {
+        if end - start != SCRIPT_LENGTH {
             return Err(ParserError::InvalidSubtag);
         }
 
-        let s = TinyStr4::from_bytes(v).map_err(|_| ParserError::InvalidSubtag)?;
-        if !s.is_ascii_alphabetic() {
-            return Err(ParserError::InvalidSubtag);
+        match TinyAsciiStr::from_bytes_manual_slice(v, start, end) {
+            Ok(s) if s.is_ascii_alphabetic() => Ok(Self(s.to_ascii_titlecase())),
+            _ => Err(ParserError::InvalidSubtag),
+        }
+    }
+
+    /// Safely creates a [`Script`] from a reference to its raw format
+    /// as returned by [`Script::into_raw()`].
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use icu::locid::subtags::Script;
+    ///
+    /// assert!(matches!(Script::try_from_raw(*b"Latn"), Ok(_)));
+    /// assert!(matches!(Script::try_from_raw(*b"Mymr"), Ok(_)));
+    /// assert!(matches!(Script::try_from_raw(*b"1234"), Err(_)));
+    ///
+    /// // Unlike the other constructors, this one is case-sensitive:
+    /// assert!(matches!(Script::try_from_raw(*b"LATN"), Err(_)));
+    /// ```
+    pub fn try_from_raw(v: [u8; 4]) -> Result<Self, ParserError> {
+        let s = TinyAsciiStr::<{ core::mem::size_of::<Self>() }>::try_from_raw(v)
+            .map_err(|_| ParserError::InvalidSubtag)?;
+        let is_valid = match s.len() {
+            SCRIPT_LENGTH => s.is_ascii_alphabetic_titlecase(),
+            _ => false,
+        };
+        if is_valid {
+            Ok(Self(s))
+        } else {
+            Err(ParserError::InvalidSubtag)
         }
-        Ok(Self(s.to_ascii_titlecase()))
     }
 
     /// Deconstructs the [`Script`] into raw format to be consumed
@@ -67,8 +107,8 @@ impl Script {
     /// let script = unsafe { Script::from_raw_unchecked(raw) };
     /// assert_eq!(script, "Latn");
     /// ```
-    pub fn into_raw(self) -> u32 {
-        self.0.into()
+    pub fn into_raw(self) -> [u8; 4] {
+        *self.0.all_bytes()
     }
 
     /// Constructor which takes a raw value returned by
@@ -89,10 +129,10 @@ impl Script {
     ///
     /// # Safety
     ///
-    /// This function accepts a [`u32`] that is expected to be a valid [`TinyStr4`]
+    /// This function accepts a [u8; 4] that is expected to be a valid [`TinyAsciiStr<4>`]
     /// representing a [`Script`] subtag in canonical syntax.
-    pub const unsafe fn from_raw_unchecked(v: u32) -> Self {
-        Self(TinyStr4::new_unchecked(v))
+    pub const unsafe fn from_raw_unchecked(v: [u8; 4]) -> Self {
+        Self(TinyAsciiStr::from_bytes_unchecked(v))
     }
 
     /// A helper function for displaying
@@ -138,7 +178,7 @@ impl<'l> From<&'l Script> for &'l str {
     }
 }
 
-impl From<Script> for TinyStr4 {
+impl From<Script> for TinyAsciiStr<4> {
     fn from(input: Script) -> Self {
         input.0
     }
