@@ -3,9 +3,10 @@
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
 use crate::parser::errors::ParserError;
+use core::cmp::Ordering;
 use core::ops::RangeInclusive;
 use core::str::FromStr;
-use tinystr::TinyStr8;
+use tinystr::TinyAsciiStr;
 
 /// A variant subtag (examples: `"macos"`, `"posix"`, `"1996"` etc.)
 ///
@@ -23,7 +24,9 @@ use tinystr::TinyStr8;
 ///
 /// [`unicode_variant_id`]: https://unicode.org/reports/tr35/#unicode_variant_id
 #[derive(Debug, PartialEq, Eq, Clone, Hash, PartialOrd, Ord, Copy)]
-pub struct Variant(TinyStr8);
+#[repr(transparent)]
+#[cfg_attr(feature = "serde", derive(serde::Serialize))]
+pub struct Variant(TinyAsciiStr<{ *VARIANT_LENGTH.end() }>);
 
 const VARIANT_LENGTH: RangeInclusive<usize> = 4..=8;
 const VARIANT_NUM_ALPHA_LENGTH: usize = 4;
@@ -42,24 +45,65 @@ impl Variant {
     ///
     /// assert_eq!(variant, "posix");
     /// ```
-    pub fn from_bytes(v: &[u8]) -> Result<Self, ParserError> {
-        let slen = v.len();
+    pub const fn from_bytes(v: &[u8]) -> Result<Self, ParserError> {
+        Self::from_bytes_manual_slice(v, 0, v.len())
+    }
 
-        if !VARIANT_LENGTH.contains(&slen) {
+    /// Equivalent to [`from_bytes(bytes[start..end])`](Self::from_bytes),
+    /// but callable in a `const` context (which range indexing is not).
+    pub const fn from_bytes_manual_slice(
+        v: &[u8],
+        start: usize,
+        end: usize,
+    ) -> Result<Self, ParserError> {
+        let slen = end - start;
+
+        if slen < *VARIANT_LENGTH.start() || *VARIANT_LENGTH.end() < slen {
             return Err(ParserError::InvalidSubtag);
         }
 
-        let s = TinyStr8::from_bytes(v).map_err(|_| ParserError::InvalidSubtag)?;
-
-        if !s.is_ascii_alphanumeric() {
+        #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
+        if slen == VARIANT_NUM_ALPHA_LENGTH && !v[start].is_ascii_digit() {
             return Err(ParserError::InvalidSubtag);
         }
 
-        if slen == VARIANT_NUM_ALPHA_LENGTH && !v[0].is_ascii_digit() {
-            return Err(ParserError::InvalidSubtag);
+        match TinyAsciiStr::from_bytes_manual_slice(v, start, end) {
+            Ok(s) if s.is_ascii_alphanumeric() => Ok(Self(s.to_ascii_lowercase())),
+            _ => Err(ParserError::InvalidSubtag),
         }
+    }
 
-        Ok(Self(s.to_ascii_lowercase()))
+    /// Safely creates a [`Variant`] from a reference to its raw format
+    /// as returned by [`Variant::into_raw()`].
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use icu::locid::subtags::Variant;
+    ///
+    /// assert!(matches!(Variant::try_from_raw(*b"fonipa\0\0"), Ok(_)));
+    /// assert!(matches!(Variant::try_from_raw(*b"1992\0\0\0\0"), Ok(_)));
+    /// assert!(matches!(Variant::try_from_raw(*b"foo\0\0\0\0\0"), Err(_)));
+    ///
+    /// // Unlike the other constructors, this one is case-sensitive:
+    /// assert!(matches!(Variant::try_from_raw(*b"POSIX\0\0\0"), Err(_)));
+    /// ```
+    pub fn try_from_raw(v: [u8; 8]) -> Result<Self, ParserError> {
+        let s = TinyAsciiStr::<{ core::mem::size_of::<Self>() }>::try_from_raw(v)
+            .map_err(|_| ParserError::InvalidSubtag)?;
+        #[allow(clippy::indexing_slicing)] // TODO(#1668) Clippy exceptions need docs or fixing.
+        let is_valid = s.is_ascii_alphanumeric()
+            && s.is_ascii_lowercase()
+            && match s.len() {
+                VARIANT_NUM_ALPHA_LENGTH => s.as_bytes()[0].is_ascii_digit(),
+                4..=8 => true, // VARIANT_LENGTH
+                _ => false,
+            };
+        if is_valid {
+            Ok(Self(s))
+        } else {
+            Err(ParserError::InvalidSubtag)
+        }
     }
 
     /// Deconstructs the [`Variant`] into raw format to be consumed
@@ -77,8 +121,8 @@ impl Variant {
     /// let variant = unsafe { Variant::from_raw_unchecked(raw) };
     /// assert_eq!(variant, "posix");
     /// ```
-    pub fn into_raw(self) -> u64 {
-        self.0.into()
+    pub fn into_raw(self) -> [u8; 8] {
+        *self.0.all_bytes()
     }
 
     /// Constructor which takes a raw value returned by
@@ -99,10 +143,10 @@ impl Variant {
     ///
     /// # Safety
     ///
-    /// This function accepts a [`u64`] that is expected to be a valid [`TinyStr8`]
+    /// This function accepts a [u8; 8] that is expected to be a valid [`TinyAsciiStr<8>`]
     /// representing a [`Variant`] subtag in canonical syntax.
-    pub const unsafe fn from_raw_unchecked(v: u64) -> Self {
-        Self(TinyStr8::new_unchecked(v))
+    pub const unsafe fn from_raw_unchecked(v: [u8; 8]) -> Self {
+        Self(TinyAsciiStr::from_bytes_unchecked(v))
     }
 
     /// A helper function for displaying
@@ -141,6 +185,11 @@ impl PartialEq<&str> for Variant {
         self.as_str() == *other
     }
 }
+impl PartialOrd<&str> for Variant {
+    fn partial_cmp(&self, other: &&str) -> Option<Ordering> {
+        Some(self.as_str().cmp(*other))
+    }
+}
 
 impl PartialEq<str> for Variant {
     fn eq(&self, other: &str) -> bool {
@@ -154,7 +203,7 @@ impl<'l> From<&'l Variant> for &'l str {
     }
 }
 
-impl From<Variant> for TinyStr8 {
+impl From<Variant> for TinyAsciiStr<8> {
     fn from(input: Variant) -> Self {
         input.0
     }
