@@ -166,6 +166,13 @@ impl Variants {
     pub fn clear(&mut self) {
         self.0 = None;
     }
+
+    pub(crate) fn for_each_subtag_str<E, F>(&self, f: &mut F) -> Result<(), E>
+    where
+        F: FnMut(&str) -> Result<(), E>,
+    {
+        self.deref().iter().map(|t| t.as_str()).try_for_each(f)
+    }
 }
 
 impl_writeable_for_subtag_list!(Variants, "macos", "posix");
