@@ -12,14 +12,14 @@ use icu_locid::extensions::Extensions;
 use icu_locid::{subtags, LanguageIdentifier, Locale, ParserError};
 use serde::Deserialize;
 
-#[derive(Deserialize, Clone)]
+#[derive(Debug, Deserialize, Clone)]
 pub struct LocaleIdentifier {
     #[serde(rename = "type")]
     pub field_type: String,
     pub identifier: String,
 }
 
-#[derive(Deserialize, Clone)]
+#[derive(Debug, Deserialize, Clone)]
 pub struct LocaleExtensionUnicode {
     #[serde(default)]
     keywords: HashMap<String, Option<String>>,
@@ -27,14 +27,14 @@ pub struct LocaleExtensionUnicode {
     attributes: Vec<String>,
 }
 
-#[derive(Deserialize, Clone)]
+#[derive(Debug, Deserialize, Clone)]
 pub struct LocaleExtensionTransform {
     tlang: Option<String>,
     #[serde(default)]
     tfields: HashMap<String, Option<String>>,
 }
 
-#[derive(Deserialize, Clone)]
+#[derive(Debug, Deserialize, Clone)]
 pub struct LocaleExtensions {
     unicode: Option<LocaleExtensionUnicode>,
     transform: Option<LocaleExtensionTransform>,
@@ -49,7 +49,7 @@ impl TryFrom<LocaleExtensions> for Extensions {
     fn try_from(input: LocaleExtensions) -> Result<Self, Self::Error> {
         let mut ext = Extensions::default();
         if let Some(unicode) = input.unicode {
-            let mut v: Vec<(unicode::Key, unicode::Value)> = unicode
+            ext.unicode.keywords = unicode
                 .keywords
                 .iter()
                 .map(|(k, v)| {
@@ -65,8 +65,6 @@ impl TryFrom<LocaleExtensions> for Extensions {
                     )
                 })
                 .collect();
-            v.sort_by_key(|i| i.0);
-            ext.unicode.keywords = unicode::Keywords::from_vec_unchecked(v);
             let v: Vec<unicode::Attribute> = unicode
                 .attributes
                 .iter()
@@ -77,7 +75,7 @@ impl TryFrom<LocaleExtensions> for Extensions {
             ext.unicode.attributes = unicode::Attributes::from_vec_unchecked(v);
         }
         if let Some(transform) = input.transform {
-            let mut v: Vec<(transform::Key, transform::Value)> = transform
+            ext.transform.fields = transform
                 .tfields
                 .iter()
                 .map(|(k, v)| {
@@ -92,8 +90,6 @@ impl TryFrom<LocaleExtensions> for Extensions {
                     )
                 })
                 .collect();
-            v.sort_by_key(|i| i.0);
-            ext.transform.fields = transform::Fields::from_vec_unchecked(v);
 
             if let Some(tlang) = transform.tlang {
                 ext.transform.lang = Some(tlang.parse().expect("Failed to parse tlang."));
@@ -109,7 +105,7 @@ impl TryFrom<LocaleExtensions> for Extensions {
     }
 }
 
-#[derive(Deserialize, Clone)]
+#[derive(Debug, Deserialize, Clone)]
 pub struct LocaleSubtags {
     #[serde(rename = "type")]
     pub field_type: String,
@@ -121,13 +117,13 @@ pub struct LocaleSubtags {
     pub extensions: Option<LocaleExtensions>,
 }
 
-#[derive(Deserialize, Clone)]
+#[derive(Debug, Deserialize, Clone)]
 pub struct LocaleError {
     pub error: String,
     pub text: String,
 }
 
-#[derive(Deserialize, Clone)]
+#[derive(Debug, Deserialize, Clone)]
 #[serde(untagged)]
 #[allow(clippy::large_enum_variant)] // test code
 pub enum LocaleInfo {
@@ -256,7 +252,7 @@ impl From<LocaleError> for ParserError {
     }
 }
 
-#[derive(Deserialize)]
+#[derive(Debug, Deserialize)]
 pub struct LocaleTest {
     pub input: LocaleInfo,
     pub output: LocaleInfo,
